@@ -49,6 +49,17 @@ function _get_buildenv(key)
     return value
 end
 
+-- get vs arch
+function _get_vsarch()
+    local arch = get_config("arch") or os.arch()
+    if arch == "x86" or arch == "i386" then return "Win32" end
+    if arch == "x86_64" then return "x64" end
+    if arch == "arm64ec" then return "ARM64EC" end
+    if arch:startswith("arm64") then return "ARM64" end
+    if arch:startswith("arm") then return "ARM" end
+    return arch
+end
+
 -- get msvc
 function _get_msvc()
     local msvc = toolchain.load("msvc")
@@ -101,8 +112,30 @@ function _is_cross_compilation()
     return false
 end
 
+function _get_cmake_system_processor()
+    -- on Windows, CMAKE_SYSTEM_PROCESSOR comes from PROCESSOR_ARCHITECTURE
+    -- on other systems it's the output of uname -m
+    if is_plat("windows") then
+        local archs = {
+            x86 = "x86",
+            x64 = "AMD64",
+            x86_64 = "AMD64",
+            arm = "ARM",
+            arm64 = "ARM64",
+            arm64ec = "ARM64EC"
+        }
+        return archs[os.subarch()] or os.subarch()
+    end
+    return os.subarch()
+end
+
 -- get configs for windows
-function _get_configs_for_windows(configs)
+function _get_configs_for_windows(configs, opt)
+    opt = opt or {}
+    local cmake_generator = opt.cmake_generator
+    if cmake_generator and not cmake_generator:find("Visual Studio", 1, true) then
+        return
+    end
     table.insert(configs, "-A")
     if is_arch("x86", "i386") then
         table.insert(configs, "Win32")
@@ -129,6 +162,13 @@ function _get_configs_for_android(configs)
         if ndk_cxxstl then
             table.insert(configs, "-DANDROID_STL=" .. ndk_cxxstl)
         end
+
+        -- avoid find and add system include/library path
+        -- @see https://github.com/xmake-io/xmake/issues/2037
+        table.insert(configs, "-DCMAKE_FIND_ROOT_PATH_MODE_PACKAGE=BOTH")
+        table.insert(configs, "-DCMAKE_FIND_ROOT_PATH_MODE_LIBRARY=BOTH")
+        table.insert(configs, "-DCMAKE_FIND_ROOT_PATH_MODE_INCLUDE=BOTH")
+        table.insert(configs, "-DCMAKE_FIND_ROOT_PATH_MODE_PROGRAM=NEVER")
     end
 end
 
@@ -154,8 +194,9 @@ function _get_configs_for_appleos(configs)
         if is_arch("x86_64", "i386") then
             envs.CMAKE_OSX_SYSROOT = "iphonesimulator"
         end
-    elseif is_plat("macosx") then
+    elseif _is_cross_compilation() then
         envs.CMAKE_SYSTEM_NAME = "Darwin"
+        envs.CMAKE_SYSTEM_PROCESSOR = _get_cmake_system_processor()
     end
     envs.CMAKE_FIND_ROOT_PATH_MODE_LIBRARY   = "BOTH"
     envs.CMAKE_FIND_ROOT_PATH_MODE_INCLUDE   = "BOTH"
@@ -187,11 +228,13 @@ function _get_configs_for_mingw(configs)
     envs.CMAKE_EXE_LINKER_FLAGS    = table.concat(table.wrap(_get_buildenv("ldflags")), ' ')
     envs.CMAKE_SHARED_LINKER_FLAGS = table.concat(table.wrap(_get_buildenv("shflags")), ' ')
     envs.CMAKE_SYSTEM_NAME         = "Windows"
+    envs.CMAKE_SYSTEM_PROCESSOR    = _get_cmake_system_processor()
     -- avoid find and add system include/library path
     envs.CMAKE_FIND_ROOT_PATH      = sdkdir
     envs.CMAKE_SYSROOT             = sdkdir
-    envs.CMAKE_FIND_ROOT_PATH_MODE_LIBRARY = "ONLY"
-    envs.CMAKE_FIND_ROOT_PATH_MODE_INCLUDE = "ONLY"
+    envs.CMAKE_FIND_ROOT_PATH_MODE_PACKAGE = "BOTH"
+    envs.CMAKE_FIND_ROOT_PATH_MODE_LIBRARY = "BOTH"
+    envs.CMAKE_FIND_ROOT_PATH_MODE_INCLUDE = "BOTH"
     envs.CMAKE_FIND_ROOT_PATH_MODE_PROGRAM = "NEVER"
     -- avoid add -isysroot on macOS
     envs.CMAKE_OSX_SYSROOT = ""
@@ -255,8 +298,9 @@ function _get_configs_for_cross(configs)
     -- avoid find and add system include/library path
     envs.CMAKE_FIND_ROOT_PATH      = sdkdir
     envs.CMAKE_SYSROOT             = sdkdir
-    envs.CMAKE_FIND_ROOT_PATH_MODE_LIBRARY = "ONLY"
-    envs.CMAKE_FIND_ROOT_PATH_MODE_INCLUDE = "ONLY"
+    envs.CMAKE_FIND_ROOT_PATH_MODE_PACKAGE = "BOTH"
+    envs.CMAKE_FIND_ROOT_PATH_MODE_LIBRARY = "BOTH"
+    envs.CMAKE_FIND_ROOT_PATH_MODE_INCLUDE = "BOTH"
     envs.CMAKE_FIND_ROOT_PATH_MODE_PROGRAM = "NEVER"
     -- avoid add -isysroot on macOS
     envs.CMAKE_OSX_SYSROOT = ""
@@ -307,8 +351,8 @@ function _get_configs_for_host_toolchain(configs)
     envs.CMAKE_SHARED_LINKER_FLAGS = table.concat(table.wrap(_get_buildenv("shflags")), ' ')
     -- we don't need to set it as cross compilation if we just pass toolchain
     -- https://github.com/xmake-io/xmake/issues/2170
-    if not is_plat(os.subhost()) then
-        envs.CMAKE_SYSTEM_NAME     = "Linux"
+    if _is_cross_compilation() then
+        envs.CMAKE_SYSTEM_NAME = "Linux"
     end
     for k, v in pairs(envs) do
         table.insert(configs, "-D" .. k .. "=" .. v)
@@ -377,7 +421,7 @@ function _get_configs(opt)
     _get_configs_for_install(configs, opt)
     _get_configs_for_generator(configs, opt)
     if is_plat("windows") then
-        _get_configs_for_windows(configs)
+        _get_configs_for_windows(configs, opt)
     elseif is_plat("android") then
         _get_configs_for_android(configs)
     elseif is_plat("iphoneos", "watchos") or
@@ -423,7 +467,9 @@ function _build_for_msvc(opt)
     local runenvs = _get_msvc_runenvs()
     local msbuild = find_tool("msbuild", {envs = runenvs})
     local slnfile = assert(find_file("*.sln", os.curdir()), "*.sln file not found!")
-    os.vexecv(msbuild.program, {slnfile, "-nologo", "-t:Build", "-m", "-p:Configuration=" .. (is_mode("debug") and "Debug" or "Release"), "-p:Platform=" .. (is_arch("x64") and "x64" or "Win32")}, {envs = runenvs})
+    os.vexecv(msbuild.program, {slnfile, "-nologo", "-t:Build", "-m",
+        "-p:Configuration=" .. (is_mode("debug") and "Debug" or "Release"),
+        "-p:Platform=" .. _get_vsarch()}, {envs = runenvs})
     local projfile = os.isfile("INSTALL.vcxproj") and "INSTALL.vcxproj" or "INSTALL.vcproj"
     if os.isfile(projfile) then
         os.vexecv(msbuild.program, {projfile, "/property:configuration=" .. (is_mode("debug") and "Debug" or "Release")}, {envs = runenvs})
